/***************************************************************************

 File Name:   aolEditableCombobox.js

****************************************************************************
C O P Y R I G H T   A N D   C O N F I D E N T I A L I T Y   N O T I C E
****************************************************************************

  Copyright (c) 2008 AOL, LLC.  All rights reserved.
  This software contains valuable confidential and proprietary
  information of AOL, LLC. and is subject to applicable licensing
  agreements.  Unauthorized reproduction, transmission or
  distribution of this file and its contents is a violation of
  applicable laws.

                      A O L    C O N F I D E N T I A L

***************************************************************************/

function component (_gadget)
{
    this.g_readOnlyMode = _gadget.getAttribute("readOnly");
    this.displayMatchesOnly = false;
    this.cancelPopping = false;
    this.focusTextOnPopupClose = true;
}
component.prototype.addItem = function(menuItem)
{
    this._gadget.insertItemAtIndex(menuItem, this._gadget.rowCount);
}
component.prototype.addItemsFromArray = function(menuItems)
{
    this._gadget.insertItemsAtIndex(menuItems, this._gadget.rowCount);
}
component.prototype.insertItemAtIndex = function(menuItem, index)
{
    if(menuItem)
    {
        if(index < 0)
        {
            index = 0;
        }
        else if(index > this._gadget.rowCount)
        {
            index = this._gadget.rowCount;
        }
        this._gadget.insertContentChildAt(menuItem, index);
        if(menuItem.getAttribute("selected") == "true" || this._gadget.selection.rowCount == 0)
        {
            this._gadget.setSelectedIndex(index);
        }
    }
}
component.prototype.insertItemsAtIndex = function(menuItems, index)
{
    if(menuItems)
    {
        if(index < 0)
        {
            index = 0;
        }
        else if(index > this._gadget.rowCount)
        {
            index = this._gadget.rowCount;
        }
        var indexToSelect = this._gadget.selectedIndex;
        var insertionPoint = index;
        for(var i=0; i<menuItems.count; i++)
        {
            var item = menuItems.valueAtIndex(i);
            this._gadget.insertContentChildAt(item, insertionPoint++);
            if(item.getAttribute("selected") == "true")
            {
                indexToSelect = i;
            }
        }
        this._gadget.setSelectedIndex(indexToSelect == -1 ? 0 : indexToSelect);
    }
}
component.prototype.removeAllItems = function()
{
    this._gadget.selection.clear();
    this._gadget.deleteContentChildren();
}
component.prototype.removeItem = function(menuItem)
{
    if(menuItem)
    {
        var parent = menuItem.parent;
        if(menuItem && parent && parent.gadgetParent == this._gadget.getPartById("thePopup"))
        {
            this._gadget.removeItemAtIndex(menuItem.parent.getChildIndex(menuItem));
        }
    }
}
component.prototype.removeItemAtIndex = function(index)
{
    this._gadget.removeItemsInRange(index, index+1);
}
component.prototype.removeItemsInRange = function(start, end)
{
    if(start >= 0 && end > 0 && end <= this._gadget.rowCount)
    {
        var currentIndex = this._gadget.selectedIndex; 
        var numTotalItems = this._gadget.rowCount;
        var numItemsToDelete = end-start;
        if(numTotalItems == numItemsToDelete)
        {
            this._gadget.removeAllItems();
            return;
        }
        var indexToSelect = currentIndex;
        if(indexToSelect == -1) {indexToSelect = 0}
        if(indexToSelect >= start)
        {
            if(start > 0)
            {
                indexToSelect = start - 1;
            }
            else
            {
                indexToSelect = end - numItemsToDelete;
            }
        }
        for(var i=0; i<numItemsToDelete; i++)
        {
            var boxToDel = this._gadget.getItemAtIndex(start);
            this._gadget.deleteContentChild(boxToDel);
        }
        this._gadget.setSelectedIndex(indexToSelect);
    }
}
component.prototype.getItemAtIndex = function(index)
{
    var item = null;
    if(index >= 0 && index < this._gadget.rowCount)
    {
        item = this._gadget.getBoxAtRow(index);
    }
    return item;
}
component.prototype.getItemsAsArray = function()
{
    var itemArray = shell.serviceManager.basics.array;
    var child = this._gadget.getItemAtIndex(0);
    while(child)
    {
        itemArray.addValue(child);
        child = child.nextSibling;
    }
    return itemArray;
}
component.prototype.setSelectedIndex = function(index)
{
    if(index >= 0 && index < this._gadget.rowCount)
    {
        this._gadget.getBoxAtRow(index).command();
    }
}
component.prototype.getSelectedIndex = function()
{
    return this._gadget.selectedIndex;
}
component.prototype.setSelectedItem = function(item)
{
    if(item && item.parent && item.parent.gadgetParent == this._gadget.getPartById("thePopup"))
    {
        item.command();
    }
}
component.prototype.getSelectedItem = function()
{
    return this._gadget.selectedBox;
}
component.prototype.selectNextComboItem = function(next)
{
    var event = shell.currentEvent;
    if(event)
    {
        event.stopPropagation();
    }
    var gadgetLabel = this._gadget.getAttribute("label");
    var popupBox = this._gadget.getPartById("thePopup");
    if (popupBox)
    {
        var child = 0;
        if (this._gadget.selectedBox == null)
        {
            child = popupBox.firstContentChild;
            while(child)
            {
                var attr1 = child.getAttribute("selectablePopupChild");
                if (attr1 == "true")
                {
                    var label = child.getAttribute("label");
                    if (label && label == gadgetLabel)
                    {
                        break;
                    }
                }
                child = child.nextSibling;
            }
        }
        else
        {
            child = this._gadget.selectedBox;
        }
        if (child)
        {
            child = next ? child.nextSibling : child.previousSibling;
            while (child)
            {
                if (!child.disabled && child.getAttribute("selectablePopupChild") == "true")
                {
                    this.getAttributesFromItem(child);
                    child.command();
                    this.requestFocus();
                    break;
                }
                child = next ? child.nextSibling : child.previousSibling;
            }
            this._gadget.selectedBox = child;
        }
    }
}
component.prototype.getAttributesFromItem = function(item)
{
    if (item)
    {
        var label = item.getAttribute("label");
        if (label)
        {
            this._gadget.setAttribute("label", label);
        }
        var value = item.getAttribute("value");
        if (value)
        {
            this._gadget.setAttribute("value", value);
        }
        var icon = item.getAttribute("icon");
        if (icon)
        {
            this._gadget.setAttribute("icon", icon);
        }
        else
        {
            this._gadget.setAttribute("icon", "");
        }
    }
}
component.prototype.onPopping = function()
{
    this.modifyDisplayResults(!this.displayMatchesOnly);
    var menuArea = this._gadget.getPartById("menuArea");
    if (menuArea)
    {
        menuArea.setAttribute("popped", "true");
    }
    this.alignPopupWithGadget();
    if(!this.displayMatchesOnly)
    {
        this.requestFocus();
    }
}
component.prototype.onPopped = function()
{
    if(this.cancelPopping)
    {
        this.hidePopup();
        return;
    }
    var popupBox = this._gadget.getPartById("thePopup");
    var child = this._gadget.selectedBox;
    if (popupBox && child)
    {                        
        var attr1 = child.getAttribute("selectablePopupChild");
        var attr2 = child.hidden;
        var attr3 = child.collapsed;
        if (attr1 == "true" && (!attr2 || attr2 == "false") && (!attr3 || attr3 == "false"))
        {
            popupBox.currentPopupChild = child;
            var container = this._gadget.getPartById("container");
            if (container && container.overflowed)
            {
                container.scrollToBox(child);
            }            
        }        
    }
}
component.prototype.onPopupClosed = function()
{
    var menuArea = this._gadget.getPartById("menuArea");
    if (menuArea)
    {
        menuArea.setAttribute("popped", "false");
    }
    this.displayMatchesOnly = false; 
    this.cancelPopping = false;
    if(this.focusTextOnPopupClose)
    {
        this.requestFocus();
    }
    this.focusTextOnPopupClose = true;
}
component.prototype.alignPopupWithGadget = function()
{
    var comboPopup = this._gadget.getPartById("thePopup");
    if (comboPopup)
    {
        comboPopup.anchor = this._gadget;
        var spaceBelowCombo = appUtils.workAreaY + appUtils.workAreaHeight - (this._gadget.screenY + this._gadget.boxHeight);
        var spaceAboveCombo = this._gadget.screenY - appUtils.workAreaY;
        if (comboPopup.boxHeight <= spaceBelowCombo)
        {
            comboPopup.top = this._gadget.boxHeight;
        }
        else if (comboPopup.boxHeight <= spaceAboveCombo)
        {
            comboPopup.top = - comboPopup.boxHeight;
        }
        else
        {
            comboPopup.top = this._gadget.boxHeight;
            comboPopup.adjustHeight(spaceBelowCombo);
        }
        comboPopup.left   = this._gadget.left;
        var container = comboPopup.getPartById("container");
        var firstContentChild = comboPopup.firstContentChild;
        if (container && container.boxWidth < this._gadget.boxWidth && firstContentChild)
        {
            firstContentChild.minWidth = this._gadget.boxWidth -
                container.getStyle("paddingLeft").intValue -
                container.getStyle("paddingRight").intValue -
                firstContentChild.getStyle("marginLeft").intValue -
                firstContentChild.getStyle("marginRight").intValue -
                firstContentChild.getStyle("paddingLeft").intValue -
                firstContentChild.getStyle("paddingRight").intValue;
            this.scene.layout(); 
        }
    }
}
component.prototype.onReadOnlyMode = function()
{
    if(this._gadget.getAttribute("readOnly") == "true")
    {
        this.g_readOnlyMode = true;
        var inputField = this._gadget.getPartById("inputField");
        if (inputField)
        {
            this._gadget.setAttribute("focusMode", "normal");
            inputField.setAttribute("focusMode", "ignore");
            inputField.setAttribute("readOnly", "true");
        }
        if(!this._gadget.disabled)
        {
            this._gadget.setAttribute("popup", "thePopup");
        }
    }
    else
    {
        this.g_readOnlyMode = false;
        var inputField = this._gadget.getPartById("inputField");
        if (inputField)
        {
            this._gadget.setAttribute("focusMode", "defer");
            inputField.setAttribute("focusMode", "normal");
            inputField.setAttribute("readOnly", "false");
        }
        this._gadget.removeAttribute("popup", false);
    }
}
component.prototype.onDisabled = function()
{
    if(this._gadget.disabled && this.g_readOnlyMode)
    {
        this._gadget.removeAttribute("popup", false);
    }
    else if(this.g_readOnlyMode)
    {
        var inputField = this._gadget.getPartById("inputField");
        if (inputField)
            inputField.disabled = false;
        var menuArea = this._gadget.getPartById("menuArea");
        this._gadget.setAttribute("popup", menuArea.getAttribute("popup"));
    }
}
component.prototype.requestFocus = function()
{
    if(!this.g_readOnlyMode)
    {
        var inputField = this._gadget.getPartById("inputField");
        if (inputField)
        {
            inputField.focus();
            inputField.selectAll();
        }
    }
    else
    {
        this.scene.focusedBox = this._gadget;
    }
}
component.prototype.showPopup = function()
{
    var popupBox = this._gadget.getPartById("thePopup");
    if (popupBox)
    {
        popupBox.opened = true;
    }
}
component.prototype.hidePopup = function(bNoFocusChange)
{
    var popupBox = this._gadget.getPartById("thePopup");
    if (popupBox)
    {
        if(bNoFocusChange)
        {
            this.focusTextOnPopupClose = false;
        }
        popupBox.opened = false;
    }
}
component.prototype.togglePopup = function()
{
    var popupBox = this._gadget.getPartById("thePopup");
    if (popupBox)
    {
        popupBox.opened = !popupBox.opened;
    }
}
component.prototype.onCommand = function()
{
    var targetBox = shell.currentEventTargetBox;
    if (targetBox)
    {
        this.getAttributesFromItem(targetBox);
        this.requestFocus();
        this._gadget.selectedBox = targetBox;
    }
}
component.prototype.updateInputField = function(isUp)
{
    var popupBox = this._gadget.getPartById("thePopup");
    if (popupBox)
    {
        if (isUp)
            popupBox.onUp();
        else
            popupBox.onDown();
    }
    var inputField = this._gadget.getPartById("inputField");
    if (popupBox && inputField)
    {
        var child = popupBox.firstPopupChild;
        while (child)
        {
            if (child.getAttribute("current") == "true" && child.getAttribute("selectablePopupChild") == "true")
            {
                this.getAttributesFromItem(child);
                this.requestFocus();
                this._gadget.selectedBox = child;
                break;
            }
            child = child.nextSibling;
        }
    }
}
component.prototype.inputLabelChanged = function()
{
    var bUseAutoSuggest = this._gadget.getAttribute("useAutoSuggest");
    var inputLabel = this._gadget.getAttribute("label");
    if(bUseAutoSuggest == "true")
    {
        if(inputLabel && inputLabel != "")
        {
            var numMatches = this.modifyDisplayResults();
            if(numMatches > 0)
            {
                this.displayMatchesOnly = true;
                this.showPopup();
            }
            else
            {
                this.hidePopup(true);
            }
        }
        else
        {
            this.modifyDisplayResults(true);
            this.hidePopup(true);
            this.displayMatchesOnly = false;
        }
    }
}
component.prototype.modifyDisplayResults = function(bIgnoreInputValue)
{
    var numMatchesFound = 0;
    var popupBox = this._gadget.getPartById("thePopup");
    var child = popupBox.firstContentChild;
    if(bIgnoreInputValue)
    {
        while(child)
        {
            if (child.getAttribute("selectablePopupChild") == "true")
            {
                child.collapsed = false;
            }
            numMatchesFound++;
            child = child.nextSibling;
        }
    }
    else
    {
        var inputLabel = this._gadget.getAttribute("label");
        if(inputLabel)
        {
            inputLabel = inputLabel.toLowerCase();
            while(child)
            {
                if (child.getAttribute("selectablePopupChild") == "true")
                {
                    var childLabel = child.getAttribute("label");
                    if(childLabel && childLabel.toLowerCase().indexOf(inputLabel) == 0)
                    {
                        child.collapsed = false;
                        numMatchesFound++;
                    }
                    else
                    {
                        child.collapsed = true;
                    }
                }
                child = child.nextSibling;
            }
        }
    }
    this.scene.layout();
    this._gadget.setAttribute("numMatchesFound", numMatchesFound);
    return numMatchesFound;
}
component.prototype.onTabPress = function()
{
    this.hidePopup();
}
component.prototype.onMouseWheel = function(detail)
{
    var comboPopup = this._gadget.getPartById("thePopup");
    var currentEvent = shell.currentEvent;
    if (comboPopup && comboPopup.opened)
    {
        var container = comboPopup.getPartById("container");
        if (container && container.overflowed)
        {
            if (detail == 1)
            {
                container.scrollBy(-12 * appUtils.mouseWheelScrollLines);
            }
            else if (detail == -1)
            {
                container.scrollBy(12 * appUtils.mouseWheelScrollLines);
            }
        }
    }
    else
    {
        if (detail == 1)
        {
            this.selectNextComboItem(false);
        }
        else if (detail == -1)
        {
            this.selectNextComboItem(true);
        }
    }
    if (currentEvent)
    {
        currentEvent.stopPropagation();
    }
}

component.prototype.onFocus = function()
{
    this._gadget.setAttribute( "inputFocused", "true");
}

component.prototype.onBlur = function()
{
    this._gadget.setAttribute( "inputFocused", "false");
}